
; test386.asm, by Erik Bouma, 05-21-1991
;              based on a bug (POPAD) found by Jeff Prothero
;              also checks 80286 POPA instruction

; test386.exe  assembled with Turbo Assembler 2.01
;              linked with Turbo Link 3.01

.386                                                       ; enable non-privileged
                                                           ; 80386 instructions
; --  declare segments

cseg           segment use16 word public 'code'
cseg           ends

dseg           segment use16 word public 'data'
dseg           ends

sseg           segment use16 para stack 'stack'
               db      512 dup (?)                         ; create small stack
sseg           ends

.8086                                                      ; 8086 instructions only

cseg           segment

               assume  cs:cseg,ds:dseg,ss:sseg

test_cpu       proc    far

start      :   mov     ax,dseg                             ; initialize
               mov     ds,ax                               ; data segment

               mov     dx,offset wrong_cpu                 ; assume no 386/486
               push    sp                                  ; test for a
               pop     ax                                  ; 8086/80186
               cmp     ax,sp                               ; or a 80x86
               jne     done                                ; not a 286,386 or 486

               mov     ax,1110000000000000b                ; set 3 hi bits
               push    ax                                  ; push on stack
               popf                                        ; and pop in flags
               pushf                                       ; save flags
               pop     ax                                  ; pop in ax
               and     ax,1110000000000000b                ; 3 hi bits still set ?
               jz      test_popa                           ; if not, it's a 286,
                                                           ; so skip the popad test
;  --  here the actual POPAD test starts

               mov     dx,offset _popad                    ; display 1st
               mov     ah,9                                ; part of
               int     21h                                 ; message
.386                                                       ; enable non-privileged
                                                           ; 80386 instructions
               mov     eax,12345678h                       ; load eax with some value
               sub     ebp,ebp                             ; clear the ebp
               sub     esi,esi                             ; and esi registers
               pushad                                      ; push all registers
               popad                                       ; pop all registers
               mov     edx,[ebp+esi]                       ; following instruction
               cmp     eax,12345678h                       ; test the eax register
                                                           ; see note #1 below
.8086                                                      ; 8086 instructions only
               mov     dx,offset _ok                       ; assume ok
               je      ok                                  ; if so ->
               mov     dx,offset _fails                    ; no, the bug was found
ok         :   mov     ah,9                                ; display rest
               int     21h                                 ; of message

;  --  here the actual POPA test starts

test_popa  :   mov     dx,offset _popa                     ; display 1st
               mov     ah,9                                ; part of
               int     21h                                 ; message

               mov     ax,1234h                            ; load ax with some value
               sub     bp,bp                               ; clear the bp
               sub     si,si                               ; and si registers
.286                                                       ; enable non-privileged
                                                           ; 80286 instructions
               pusha                                       ; push all registers
               popa                                        ; pop all registers
.8086                                                      ; 8086 instructions only
               mov     dx,[bp+si]                          ; following instruction
               cmp     ax,1234h                            ; test the ax register
                                                           ; see note #2 below
               mov     dx,offset _ok                       ; assume ok
               je      done                                ; if so ->
               mov     dx,offset _fails                    ; no, the bug was found

done       :   mov     ah,9                                ; display rest
               int     21h                                 ; of message
               mov     ax,4C00h                            ; all done,
               int     21h                                 ; exit

test_cpu       endp

cseg           ends

dseg           segment
_popad         db      '80386 POPAD instruction $'
_popa          db      '80286 POPA  instruction $'
_fails         db      'fails test.',7,13,10,'$'
_ok            db      'ok.',13,10,'$'
wrong_cpu      db      'Not a 80286, 80386(SX) or 80486 CPU.',7,13,10,'$'
dseg           ends

end            start






note 1 :  the instruction following the POPAD does not affect the EAX
          register, so in the compare that follows, EAX should still
          contain 12345678h. if the bug is present however, the EAX
          register is cleared and the original value (12345678h) is lost.

note 2 :  the instruction following the POPA does not affect the AX
          register, so in the compare that follows, AX should still
          contain 1234h. if the bug is present however, the AX register
          is cleared and the original value (1234h) is lost.
